@@ -58,7 +58,6 @@ typedef struct {
 	GDBusConnection *d_connection;
 	GDBusNodeInfo *introspection_data;
 	guint registration_id;
-	guint own_id;
 	gchar *full_name;
 	gchar *full_path;
 } TrackerMinerFilesIndexPrivate;
@@ -79,7 +78,6 @@ static void     index_get_property        (GObject              *object,
                                            GValue               *value,
                                            GParamSpec           *pspec);
 static void     index_finalize            (GObject              *object);
-static void     index_constructed         (GObject              *object);
 
 G_DEFINE_TYPE(TrackerMinerFilesIndex, tracker_miner_files_index, G_TYPE_OBJECT)
 
@@ -93,7 +91,6 @@ tracker_miner_files_index_class_init (TrackerMinerFilesIndexClass *klass)
 	object_class->finalize = index_finalize;
 	object_class->set_property = index_set_property;
 	object_class->get_property = index_get_property;
-	object_class->constructed  = index_constructed;
 
 	g_object_class_install_property (object_class,
 	                                 PROP_FILES_MINER,
@@ -152,10 +149,6 @@ index_finalize (GObject *object)
 {
 	TrackerMinerFilesIndexPrivate *priv = TRACKER_MINER_FILES_INDEX_GET_PRIVATE (object);
 
-	if (priv->own_id != 0) {
-		g_bus_unown_name (priv->own_id);
-	}
-
 	if (priv->registration_id != 0) {
 		g_dbus_connection_unregister_object (priv->d_connection,
 		                                     priv->registration_id);
@@ -516,10 +509,18 @@ handle_set_property (GDBusConnection  *connection,
 }
 
 static void
-index_constructed (GObject *miner)
+tracker_miner_files_index_init (TrackerMinerFilesIndex *object)
 {
+}
+
+TrackerMinerFilesIndex *
+tracker_miner_files_index_new (TrackerMinerFiles *miner_files)
+{
+	GObject *miner;
 	TrackerMinerFilesIndexPrivate *priv;
 	gchar *full_path, *full_name;
+	GVariant *reply;
+	guint32 rval;
 	GError *error = NULL;
 	GDBusInterfaceVTable interface_vtable = {
 		handle_method_call,
@@ -527,6 +528,10 @@ index_constructed (GObject *miner)
 		handle_set_property
 	};
 
+	miner = g_object_new (TRACKER_TYPE_MINER_FILES_INDEX,
+	                      "files-miner", miner_files,
+	                      NULL);
+
 	priv = TRACKER_MINER_FILES_INDEX_GET_PRIVATE (miner);
 
 	priv->d_connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
@@ -535,17 +540,19 @@ index_constructed (GObject *miner)
 		g_critical ("Could not connect to the D-Bus session bus, %s",
 		            error ? error->message : "no error given.");
 		g_clear_error (&error);
-		return;
+		g_object_unref (miner);
+		return NULL;
 	}
 
-	priv->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
+	priv->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, &error);
+	if (!priv->introspection_data) {
+		g_critical ("Could not create node info from introspection XML, %s",
+		            error ? error->message : "no error given.");
+		g_clear_error (&error);
+		return NULL;
+	}
 
 	full_name = g_strconcat (TRACKER_MINER_DBUS_NAME_PREFIX, "Files.Index", NULL);
-
-	priv->own_id = g_bus_own_name_on_connection (priv->d_connection,
-	                                             full_name,
-	                                             G_BUS_NAME_OWNER_FLAGS_NONE,
-	                                             NULL, NULL, NULL, NULL);
 	priv->full_name = full_name;
 
 	/* Register the service name for the miner */
@@ -569,21 +576,40 @@ index_constructed (GObject *miner)
 		            full_path,
 		            error ? error->message : "no error given.");
 		g_clear_error (&error);
-		return;
+		g_object_unref (miner);
+		return NULL;
+	}
+
+	reply = g_dbus_connection_call_sync (priv->d_connection,
+	                                     "org.freedesktop.DBus",
+	                                     "/org/freedesktop/DBus",
+	                                     "org.freedesktop.DBus",
+	                                     "RequestName",
+	                                     g_variant_new ("(su)", full_name, 0x4 /* DBUS_NAME_FLAG_DO_NOT_QUEUE */),
+	                                     G_VARIANT_TYPE ("(u)"),
+	                                     0, -1, NULL, &error);
+
+	if (error) {
+		g_critical ("Could not acquire name:'%s', %s",
+		            full_name,
+		            error->message);
+		g_clear_error (&error);
+		g_object_unref (miner);
+		return NULL;
+	}
+
+	g_variant_get (reply, "(u)", &rval);
+	g_variant_unref (reply);
+
+	if (rval != 1 /* DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER */) {
+		g_critical ("D-Bus service name:'%s' is already taken, "
+		            "perhaps the daemon is already running?",
+		            full_name);
+		g_object_unref (miner);
+		return NULL;
 	}
 
 	priv->full_path = full_path;
-}
 
-static void
-tracker_miner_files_index_init (TrackerMinerFilesIndex *object)
-{
-}
-
-TrackerMinerFilesIndex *
-tracker_miner_files_index_new (TrackerMinerFiles *miner_files)
-{
-	return g_object_new (TRACKER_TYPE_MINER_FILES_INDEX,
-	                     "files-miner", miner_files,
-	                     NULL);
+	return (TrackerMinerFilesIndex *) miner;
 }
