@@ -810,9 +810,10 @@ set_up_mount_point (TrackerMinerFiles *miner,
                                                 "INSERT INTO <%s> { <%s> a tracker:Volume; tracker:isMounted true } ",
                                                 removable_device_urn, removable_device_urn);
                 }
-		g_string_append_printf (queries,
-		                        "INSERT { GRAPH <%s> { ?do tracker:available true } } WHERE { ?do nie:dataSource <%s> OPTIONAL { ?do tracker:available ?available } FILTER (!bound(?available)) } ",
-		                        removable_device_urn, removable_device_urn);
+
+                g_string_append_printf (queries,
+                                        "INSERT { GRAPH <%s> { ?do tracker:available true } } WHERE { ?do nie:dataSource <%s> } ",
+                                        removable_device_urn, removable_device_urn);
 	} else {
 		gchar *now;
 
@@ -838,8 +839,9 @@ set_up_mount_point (TrackerMinerFiles *miner,
 		                        removable_device_urn, removable_device_urn);
 
 		g_string_append_printf (queries,
-		                        "DELETE { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> ; tracker:available true } ",
+		                        "DELETE { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> } ",
 		                        removable_device_urn);
+
 		g_free (now);
 	}
 
@@ -1027,6 +1029,16 @@ init_mount_points (TrackerMinerFiles *miner_files)
 				                         TRACKER_STORAGE_TYPE_IS_OPTICAL (type),
 				                         accumulator);
 
+				if (mount_point) {
+					GFile *file;
+
+					/* Add the current mount point as reported to have incorrect
+					 * state. We will force mtime checks on this mount points,
+					 * even if no-mtime-check-needed was set. */
+					file = g_file_new_for_path (mount_point);
+					tracker_miner_fs_force_mtime_checking (TRACKER_MINER_FS (miner), file);
+					g_object_unref (file);
+				}
 			}
 		} else if (!(state & VOLUME_MOUNTED) &&
 		           (state & VOLUME_MOUNTED_IN_STORE)) {
@@ -1040,6 +1052,8 @@ init_mount_points (TrackerMinerFiles *miner_files)
                                                     NULL,
 				                    FALSE,
 				                    accumulator);
+				/* There's no need to force mtime check in these inconsistent
+				 * mount points, as they are not mounted right now. */
 			}
 		}
 	}
@@ -2025,10 +2039,16 @@ extractor_get_failsafe_metadata_cb (GObject      *object,
 		g_error_free (error);
 		g_free (uri);
 	} else {
+		TrackerSparqlBuilder *builder;
+
 		g_debug ("  Extraction succeeded the second time");
 
-		preupdate = tracker_extract_info_get_preupdate (info);
-		sparql = tracker_extract_info_get_update (info);
+		builder = tracker_extract_info_get_preupdate_builder (info);
+		preupdate = tracker_sparql_builder_get_result (builder);
+
+		builder = tracker_extract_info_get_metadata_builder (info);
+		sparql = tracker_sparql_builder_get_result (builder);
+
 		where = tracker_extract_info_get_where_clause (info);
 	}
 
@@ -2069,6 +2089,7 @@ extractor_process_failsafe (TrackerMinerFiles *miner)
 
 		tracker_extract_client_get_metadata (data->file,
 						     data->mime_type,
+		                                     TRACKER_MINER_FS_GRAPH_URN,
 						     data->cancellable,
 						     extractor_get_failsafe_metadata_cb,
 						     data);
@@ -2117,7 +2138,8 @@ extractor_get_embedded_metadata_cb (GObject      *object,
 	TrackerMinerFilesPrivate *priv;
 	TrackerMinerFiles *miner;
 	ProcessFileData *data = user_data;
-	const gchar *preupdate, *sparql, *where;
+	TrackerSparqlBuilder *preupdate, *sparql;
+	const gchar *where;
 	TrackerExtractInfo *info;
 	GError *error = NULL;
 
@@ -2155,11 +2177,14 @@ extractor_get_embedded_metadata_cb (GObject      *object,
 
 		g_error_free (error);
 	} else {
-		preupdate = tracker_extract_info_get_preupdate (info);
-		sparql = tracker_extract_info_get_update (info);
+		preupdate = tracker_extract_info_get_preupdate_builder (info);
+		sparql = tracker_extract_info_get_metadata_builder (info);
 		where = tracker_extract_info_get_where_clause (info);
 
-		sparql_builder_finish (data, preupdate, sparql, where);
+		sparql_builder_finish (data,
+		                       tracker_sparql_builder_get_result (preupdate),
+		                       tracker_sparql_builder_get_result (sparql),
+		                       where);
 
 		/* Notify about the success */
 		tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, NULL);
@@ -2271,6 +2296,7 @@ process_file_cb (GObject      *object,
 		/* Next step, if handled by the extractor, get embedded metadata */
 		tracker_extract_client_get_metadata (data->file,
 		                                     mime_type,
+		                                     TRACKER_MINER_FS_GRAPH_URN,
 		                                     data->cancellable,
 		                                     extractor_get_embedded_metadata_cb,
 		                                     data);
