@@ -138,20 +138,12 @@ miner_pause (const gchar *miner,
 	str = g_strdup_printf (_("Cookie is %d"), cookie);
 	g_print ("  %s\n", str);
 	g_free (str);
+	g_object_unref (manager);
 
 	if (for_process) {
-		GMainLoop *main_loop;
-
 		g_print ("%s\n", _("Press Ctrl+C to end pause"));
-
-		main_loop = g_main_loop_new (NULL, FALSE);
-		/* Block until Ctrl+C */
-		g_main_loop_run (main_loop);
-		g_object_unref (main_loop);
 	}
 
-	g_object_unref (manager);
-
 	return EXIT_SUCCESS;
 }
 
@@ -283,11 +275,8 @@ miner_list (gboolean available,
 
 		miners_available = tracker_miner_manager_get_available (manager);
 
-		str = g_strdup_printf (ngettext ("Found %d miner installed",
-		                                 "Found %d miners installed",
-		                                 g_slist_length (miners_available)),
+		str = g_strdup_printf (_("Found %d miners installed"),
 		                       g_slist_length (miners_available));
-
 		g_print ("%s%s\n", str, g_slist_length (miners_available) > 0 ? ":" : "");
 		g_free (str);
 
@@ -306,11 +295,8 @@ miner_list (gboolean available,
 
 		miners_running = tracker_miner_manager_get_running (manager);
 
-		str = g_strdup_printf (ngettext ("Found %d miner running",
-		                                 "Found %d miners running",
-		                                 g_slist_length (miners_running)),
+		str = g_strdup_printf (_("Found %d miners running"),
 		                       g_slist_length (miners_running));
-
 		g_print ("%s%s\n", str, g_slist_length (miners_running) > 0 ? ":" : "");
 		g_free (str);
 
@@ -374,12 +360,6 @@ miner_pause_details (void)
 		                                 &pause_applications,
 		                                 &pause_reasons);
 
-		if (!pause_applications || !pause_reasons) {
-			/* unable to get pause details,
-			   already logged by tracker_miner_manager_is_paused */
-			continue;
-		}
-
 		if (!(*pause_applications) || !(*pause_reasons)) {
 			g_strfreev (pause_applications);
 			g_strfreev (pause_reasons);
@@ -466,7 +446,20 @@ tracker_control_miners_run (void)
 	}
 
 	if (pause_for_process_reason) {
-		return miner_pause (miner_name, pause_for_process_reason, TRUE);
+		gint retval;
+
+		retval = miner_pause (miner_name, pause_for_process_reason, TRUE);
+
+		if (retval == EXIT_SUCCESS) {
+			GMainLoop *main_loop;
+
+			main_loop = g_main_loop_new (NULL, FALSE);
+			/* Block until Ctrl+C */
+			g_main_loop_run (main_loop);
+			g_object_unref (main_loop);
+		}
+
+		return retval;
 	}
 
 	if (resume_cookie != -1) {
