@@ -2060,13 +2060,9 @@ parse_id3v2 (const gchar          *data,
 }
 
 G_MODULE_EXPORT gboolean
-tracker_extract_get_metadata (const gchar          *uri,
-                              const gchar          *mimetype,
-                              TrackerSparqlBuilder *preupdate,
-                              TrackerSparqlBuilder *metadata,
-                              GString              *where)
+tracker_extract_get_metadata (TrackerExtractInfo *info)
 {
-	gchar *filename;
+	gchar *filename, *uri;
 	int fd;
 	void *buffer;
 	void *id3v1_buffer;
@@ -2074,8 +2070,16 @@ tracker_extract_get_metadata (const gchar          *uri,
 	goffset  buffer_size;
 	goffset audio_offset;
 	MP3Data md = { 0 };
+	TrackerSparqlBuilder *metadata, *preupdate;
+	GFile *file;
+	const gchar *graph;
 
-	filename = g_filename_from_uri (uri, NULL, NULL);
+	graph = tracker_extract_info_get_graph (info);
+	metadata = tracker_extract_info_get_metadata_builder (info);
+	preupdate = tracker_extract_info_get_preupdate_builder (info);
+
+	file = tracker_extract_info_get_file (info);
+	filename = g_file_get_path (file);
 
 	size = tracker_file_get_size (filename);
 
@@ -2136,7 +2140,23 @@ tracker_extract_get_metadata (const gchar          *uri,
 
 	g_free (id3v1_buffer);
 
+	if (md.id3v1.encoding != NULL) {
+		gchar *locale;
+
+		locale = tracker_locale_get (TRACKER_LOCALE_LANGUAGE);
+		if (!g_str_has_prefix (locale, "ru") &&
+		    !g_str_has_prefix (locale, "uk")) {
+			/* use guessed encoding for ID3v2 tags only in selected locales
+			   where broken ID3v2 is widespread */
+			g_free (md.id3v1.encoding);
+			md.id3v1.encoding = NULL;
+		}
+		g_free (locale);
+		locale = NULL;
+	}
+
 	/* Get other embedded tags */
+	uri = g_file_get_uri (file);
 	audio_offset = parse_id3v2 (buffer, buffer_size, &md.id3v1, uri, metadata, &md);
 
 	md.title = tracker_coalesce_strip (4, md.id3v24.title2,
@@ -2240,6 +2260,9 @@ tracker_extract_get_metadata (const gchar          *uri,
 		md.performer_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", md.performer);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
+		if (graph) {
+			tracker_sparql_builder_graph_open (preupdate, graph);
+		}
 
 		tracker_sparql_builder_subject_iri (preupdate, md.performer_uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
@@ -2247,6 +2270,9 @@ tracker_extract_get_metadata (const gchar          *uri,
 		tracker_sparql_builder_predicate (preupdate, "nmm:artistName");
 		tracker_sparql_builder_object_unvalidated (preupdate, md.performer);
 
+		if (graph) {
+			tracker_sparql_builder_graph_close (preupdate);
+		}
 		tracker_sparql_builder_insert_close (preupdate);
 	}
 
@@ -2254,6 +2280,9 @@ tracker_extract_get_metadata (const gchar          *uri,
 		md.composer_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", md.composer);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
+		if (graph) {
+			tracker_sparql_builder_graph_open (preupdate, graph);
+		}
 
 		tracker_sparql_builder_subject_iri (preupdate, md.composer_uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
@@ -2261,6 +2290,9 @@ tracker_extract_get_metadata (const gchar          *uri,
 		tracker_sparql_builder_predicate (preupdate, "nmm:artistName");
 		tracker_sparql_builder_object_unvalidated (preupdate, md.composer);
 
+		if (graph) {
+			tracker_sparql_builder_graph_close (preupdate);
+		}
 		tracker_sparql_builder_insert_close (preupdate);
 	}
 
@@ -2268,11 +2300,19 @@ tracker_extract_get_metadata (const gchar          *uri,
 		md.lyricist_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", md.lyricist);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
+		if (graph) {
+			tracker_sparql_builder_graph_open (preupdate, graph);
+		}
+
 		tracker_sparql_builder_subject_iri (preupdate, md.lyricist_uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
 		tracker_sparql_builder_object (preupdate, "nmm:Artist");
 		tracker_sparql_builder_predicate (preupdate, "nmm:artistName");
 		tracker_sparql_builder_object_unvalidated (preupdate, md.lyricist);
+
+		if (graph) {
+			tracker_sparql_builder_graph_close (preupdate);
+		}
 		tracker_sparql_builder_insert_close (preupdate);
 	}
 
@@ -2280,6 +2320,9 @@ tracker_extract_get_metadata (const gchar          *uri,
 		md.album_uri = tracker_sparql_escape_uri_printf ("urn:album:%s", md.album);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
+		if (graph) {
+			tracker_sparql_builder_graph_open (preupdate, graph);
+		}
 
 		tracker_sparql_builder_subject_iri (preupdate, md.album_uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
@@ -2295,6 +2338,9 @@ tracker_extract_get_metadata (const gchar          *uri,
 			tracker_sparql_builder_object_iri (preupdate, md.performer_uri);
 		}
 
+		if (graph) {
+			tracker_sparql_builder_graph_close (preupdate);
+		}
 		tracker_sparql_builder_insert_close (preupdate);
 
 		if (md.track_count > 0) {
@@ -2310,11 +2356,17 @@ tracker_extract_get_metadata (const gchar          *uri,
 			tracker_sparql_builder_where_close (preupdate);
 
 			tracker_sparql_builder_insert_open (preupdate, NULL);
+			if (graph) {
+				tracker_sparql_builder_graph_open (preupdate, graph);
+			}
 
 			tracker_sparql_builder_subject_iri (preupdate, md.album_uri);
 			tracker_sparql_builder_predicate (preupdate, "nmm:albumTrackCount");
 			tracker_sparql_builder_object_int64 (preupdate, md.track_count);
 
+			if (graph) {
+				tracker_sparql_builder_graph_close (preupdate);
+			}
 			tracker_sparql_builder_insert_close (preupdate);
 		}
 	}
@@ -2421,6 +2473,10 @@ tracker_extract_get_metadata (const gchar          *uri,
 		tracker_sparql_builder_where_close (preupdate);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
+		if (graph) {
+			tracker_sparql_builder_graph_open (preupdate, graph);
+		}
+
 		tracker_sparql_builder_subject_iri (preupdate, album_disc_uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
 		tracker_sparql_builder_object (preupdate, "nmm:MusicAlbumDisc");
@@ -2428,6 +2484,10 @@ tracker_extract_get_metadata (const gchar          *uri,
 		tracker_sparql_builder_object_int64 (preupdate, md.set_number > 0 ? md.set_number : 1);
 		tracker_sparql_builder_predicate (preupdate, "nmm:albumDiscAlbum");
 		tracker_sparql_builder_object_iri (preupdate, md.album_uri);
+
+		if (graph) {
+			tracker_sparql_builder_graph_close (preupdate);
+		}
 		tracker_sparql_builder_insert_close (preupdate);
 
 		tracker_sparql_builder_predicate (metadata, "nmm:musicAlbumDisc");
@@ -2460,6 +2520,7 @@ tracker_extract_get_metadata (const gchar          *uri,
 #endif
 
 	g_free (filename);
+	g_free (uri);
 
 	return TRUE;
 }
