@@ -19,21 +19,9 @@
 
 #include "config.h"
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
-
-#include <errno.h>
-#include <fcntl.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <sys/mman.h>
 
 #include <glib.h>
-#include <glib/gstdio.h>
-
 #include <gio/gio.h>
 
 #include <libtracker-extract/tracker-extract.h>
@@ -41,43 +29,32 @@
 #include "tracker-main.h"
 #include "tracker-read.h"
 
-#define  TRY_LOCALE_TO_UTF8_CONVERSION 0
-
-static void extract_text (const gchar          *uri,
-                          TrackerSparqlBuilder *preupdate,
-                          TrackerSparqlBuilder *metadata);
-
-static TrackerExtractData data[] = {
-	{ "text/*", extract_text },
-	{ NULL, NULL }
-};
-
+#define TRY_LOCALE_TO_UTF8_CONVERSION 0
 
 static gchar *
-get_file_content (const gchar  *uri,
-                  gsize         n_bytes)
+get_file_content (GFile *file,
+                  gsize  n_bytes)
 {
-	gchar *text, *path;
-	int fd;
+	GFileInputStream *stream;
+	GError *error = NULL;
+	gchar *text, *uri;
 
 	/* If no content requested, return */
 	if (n_bytes == 0) {
 		return NULL;
 	}
 
+	uri = g_file_get_uri (file);
+
 	/* Get filename from URI */
-	path = g_filename_from_uri (uri, NULL, NULL);
-
-	fd = g_open (path, O_RDONLY | O_NOATIME, 0);
-	if (fd == -1 && errno == EPERM) {
-		fd = g_open (path, O_RDONLY, 0);
-	}
-
-	if (fd == -1) {
-		g_message ("Could not open file '%s': %s",
+	stream = g_file_read (file, NULL, &error);
+	if (error) {
+		g_message ("Could not read file '%s': %s",
 		           uri,
-		           g_strerror (errno));
-		g_free (path);
+		           error->message);
+		g_error_free (error);
+		g_free (uri);
+
 		return NULL;
 	}
 
@@ -85,27 +62,35 @@ get_file_content (const gchar  *uri,
 	         uri, n_bytes);
 
 	/* Read up to n_bytes from stream. Output is always, always valid UTF-8 */
-	text = tracker_read_text_from_fd (fd,
-	                                  n_bytes,
-	                                  TRY_LOCALE_TO_UTF8_CONVERSION);
+	text = tracker_read_text_from_stream (G_INPUT_STREAM (stream),
+	                                      n_bytes,
+	                                      TRY_LOCALE_TO_UTF8_CONVERSION);
 
-	close (fd);
-	g_free (path);
+	g_object_unref (stream);
+	g_free (uri);
 
 	return text;
 }
 
-static void
-extract_text (const gchar          *uri,
-              TrackerSparqlBuilder *preupdate,
-              TrackerSparqlBuilder *metadata)
+G_MODULE_EXPORT gboolean
+tracker_extract_module_init (TrackerModuleThreadAwareness  *thread_awareness_ret,
+                             GError                       **error)
 {
+	*thread_awareness_ret = TRACKER_MODULE_MULTI_THREAD;
+	return TRUE;
+}
+
+G_MODULE_EXPORT gboolean
+tracker_extract_get_metadata (TrackerExtractInfo *info)
+{
+	TrackerSparqlBuilder *metadata;
 	TrackerConfig *config;
 	gchar *content;
 
 	config = tracker_main_get_config ();
+	metadata = tracker_extract_info_get_metadata_builder (info);
 
-	content = get_file_content (uri,
+	content = get_file_content (tracker_extract_info_get_file (info),
 	                            tracker_config_get_max_bytes (config));
 
 	tracker_sparql_builder_predicate (metadata, "a");
@@ -116,10 +101,6 @@ extract_text (const gchar          *uri,
 		tracker_sparql_builder_object_unvalidated (metadata, content);
 		g_free (content);
 	}
-}
 
-TrackerExtractData *
-tracker_extract_get_data (void)
-{
-	return data;
+	return TRUE;
 }
