@@ -90,7 +90,7 @@ check_content_in_db (gint expected_instances, gint expected_relations)
 static void
 test_backup_and_restore_helper (gboolean journal)
 {
-	gchar  *data_prefix, *data_filename, *backup_filename, *db_location, *meta_db;
+	gchar  *data_prefix, *data_filename, *backup_location, *backup_filename, *db_location, *meta_db;
 	GError *error = NULL;
 	GFile  *backup_file;
 	gchar *test_schemas[5] = { NULL, NULL, NULL, NULL, NULL };
@@ -112,7 +112,8 @@ test_backup_and_restore_helper (gboolean journal)
 
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           (const gchar **) test_schemas,
-	                           NULL, FALSE, 100, 100, NULL, NULL, NULL, &error);
+	                           NULL, FALSE, FALSE,
+	                           100, 100, NULL, NULL, NULL, &error);
 
 	g_assert_no_error (error);
 
@@ -130,11 +131,12 @@ test_backup_and_restore_helper (gboolean journal)
 	/* Check everything is correct */
 	check_content_in_db (3, 1);
 
-	backup_filename = g_build_path (G_DIR_SEPARATOR_S, 
-	                                db_location, "tracker.dump",
-	                                NULL);
+	backup_location = g_build_filename (db_location, "backup", NULL);
+	g_mkdir (backup_location, 0777);
+	backup_filename = g_build_filename (backup_location, "tracker.dump", NULL);
 	backup_file = g_file_new_for_path (backup_filename);
 	g_free (backup_filename);
+	g_free (backup_location);
 	tracker_data_backup_save (backup_file,
 	                          backup_finished_cb,
 	                          NULL,
@@ -152,6 +154,7 @@ test_backup_and_restore_helper (gboolean journal)
 	g_unlink (meta_db);
 	g_free (meta_db);
 
+#ifndef DISABLE_JOURNAL
 	if (!journal) {
 		meta_db = g_build_path (G_DIR_SEPARATOR_S, db_location, "data", "tracker-store.journal", NULL);
 		g_unlink (meta_db);
@@ -161,16 +164,20 @@ test_backup_and_restore_helper (gboolean journal)
 		g_unlink (meta_db);
 		g_free (meta_db);
 	}
+#endif /* DISABLE_JOURNAL */
 
 	meta_db = g_build_path (G_DIR_SEPARATOR_S, db_location, "data", ".meta.isrunning", NULL);
 	g_unlink (meta_db);
 	g_free (meta_db);
 
+#ifndef DISABLE_JOURNAL
 	tracker_db_journal_set_rotating (FALSE, G_MAXSIZE, NULL);
+#endif /* DISABLE_JOURNAL */
 
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           (const gchar **) test_schemas,
-	                           NULL, FALSE, 100, 100, NULL, NULL, NULL, &error);
+	                           NULL, FALSE, FALSE,
+	                           100, 100, NULL, NULL, NULL, &error);
 
 	g_assert_no_error (error);
 
@@ -222,7 +229,6 @@ main (int argc, char **argv)
 
 	g_setenv ("XDG_DATA_HOME", current_dir, TRUE);
 	g_setenv ("XDG_CACHE_HOME", current_dir, TRUE);
-	g_setenv ("TRACKER_DB_SQL_DIR", TOP_SRCDIR "/data/db/", TRUE);
 	g_setenv ("TRACKER_DB_ONTOLOGIES_DIR", TOP_SRCDIR "/data/ontologies/", TRUE);
 
 	g_free (current_dir);
